// MotorController.cpp
#include "MotorController.h"

const MotorConfig defaultConfig = {
    .minPWM = 0,
    .maxPWM = 255,
    .inverted = false,
    .responseSpeed = 10,
    .lookupTable = {0, 28, 57, 85, 114, 142, 171, 199, 228, 255}
};

MotorController::MotorController(int motorPin)
    : motorPWMPin(motorPin), currentSpeed(0) {
    loadDefaults();
    pinMode(motorPWMPin, OUTPUT);
    stopMotor();
}

void MotorController::loadDefaults() {
    config = defaultConfig;
}

void MotorController::setTargetSpeed(int targetSpeedPercent) {
    targetSpeedPercent = constrain(targetSpeedPercent, 0, 100);
    int tableIndex = map(targetSpeedPercent, 0, 100, 0, MotorConfig::lookupTableSize - 1);
    uint8_t targetPWM = config.lookupTable[tableIndex];
    uint8_t adjustedPWM = applyDynamicResponse(targetPWM);
    setPWM(adjustedPWM);
    currentSpeed = targetSpeedPercent;
}

int MotorController::getSpeed() const {
    return currentSpeed;
}

void MotorController::stopMotor() {
    setTargetSpeed(0);
}

MotorConfig MotorController::getConfig() const {
    return config;
}

void MotorController::setConfig(const MotorConfig &newConfig) {
    config = newConfig;
}

void MotorController::setPWM(uint8_t pwmValue) {
    uint8_t constrainedPWM = constrain(pwmValue, config.minPWM, config.maxPWM);
    analogWrite(motorPWMPin, config.inverted ? 255 - constrainedPWM : constrainedPWM);
}

int MotorController::applyDynamicResponse(int targetValue) {
    static int smoothedValue = 0;
    int delta = targetValue - smoothedValue;
    int step = delta / config.responseSpeed;
    smoothedValue += step;
    return smoothedValue;
}
